#include <qpainter.h>
#include <kdebug.h>

#include "LTable.h"

LTable::LTable(VectorDataSource *dSource, QWidget *parent, const char *name)
        : QTable(dSource->numRows(), dSource->numCols(), parent, name),
          dataSource(dSource), editor(0)
{
    connect(dataSource, SIGNAL(dataChanged()), this, SLOT(updateContents()));
    dataSource->connectToTable(TRUE);
}

QString LTable::text(int row, int col) const {
    return dataSource->cell(row, col);
}

void LTable::setText(int row, int col, const QString &text) {
	dataSource->setCell(row,col,text);
}

void LTable::paintCell(QPainter *painter, int row, int col, const QRect &cr, bool selected, const QColorGroup &cg)
{
    QRect rect(0, 0, cr.width(), cr.height());
    if (selected) {
        painter->fillRect(rect, cg.highlight());
        painter->setPen(cg.highlightedText());
    } else {
        painter->fillRect(rect, cg.base());
        painter->setPen(QPen(Qt::lightGray, 0));
        painter->drawLine(rect.bottomLeft(), rect.bottomRight());
        painter->drawLine(rect.bottomRight(), rect.topRight());
        painter->setPen(cg.text());
    }
    painter->drawText(0, 0, cr.width(), cr.height(), AlignCenter, text(row, col));
}

QWidget *LTable::createEditor(int row, int col, bool initFromCell) const
{
    editor = new QLineEdit(viewport());
    if (initFromCell)
        editor->setText(text(row, col));
    return editor;
}

QWidget *LTable::cellWidget(int row, int col) const
{
    if (row == currEditRow() && col == currEditCol())
        return editor;
    return 0;
}

void LTable::endEdit(int row, int col, bool accept, bool replace)
{
    QTable::endEdit(row, col, accept, replace);
    delete editor;
    editor = 0;
}

void LTable::setCellContentFromEditor(int row, int col)
{
    if (editor)
        dataSource->setCell(row, col, editor->text());
}

