//LabPlot : ListDialog.cc

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <qstring.h>
#include <qfiledialog.h>
#include <klocale.h>
#include "ListDialog.h"
#include "DumpDialog.h"
#include "EditDialog.h"
#include "InfoDialog.h"
#include "VTKDialog.h"
#include "GRASSDialog.h"
#include "DataDialog.h"
#include "FunctionDialog.h"
#include "LTableItem.h"

ListDialog::ListDialog(MainWin *mw, const char *name)
	: Dialog(mw, name)
{
	if(p) {
		type = p->getPlot(p->API())->Type();

		lv = new GraphListView(vbox);

		// popup menu
		menu = new QPopupMenu( lv );
		menu->insertItem( i18n( "Show/Hide" ),this,SLOT(toggleShown()) );
		menu->insertItem( i18n( "Delete" ),this,SLOT(deleteGraph()) );
		menu->insertItem( i18n( "Change" ),this,SLOT(changeGraph()) );
		menu->insertItem( i18n( "Clone" ),this,SLOT(Clone()) );
		menu->insertSeparator();
		menu->insertItem( i18n( "Edit" ),this,SLOT(editGraph()) );
		menu->insertItem( i18n( "Open Spreadsheet" ),this,SLOT(openSpreadsheet()) );
		menu->insertItem( i18n( "Export" ),this,SLOT(dumpGraph()) );
		menu->insertItem( i18n( "Statistics" ),this,SLOT(statGraph()) );
		menu->insertSeparator();
		menu->insertItem( i18n( "Toggle Masking" ),this,SLOT(toggleMask()) );
		menu->insertItem( i18n( "Unmask" ),this,SLOT(unMask()) );

		// result selection
		QHBox *hb = new QHBox(vbox);
		new QLabel(i18n("add result to "),hb);
		sheetcb = new KComboBox(hb);
		QStringList wlist;
		QWidgetList list = mw->getWorkspace()->windowList();
		for(unsigned int i=0;i<list.count();i++)
			wlist << list.at(i)->caption();
		wlist<<i18n("new Worksheet")<<i18n("new Spreadsheet");
		sheetcb->insertStringList(wlist);
		sheetcb->setCurrentItem(0);
	
		connect(lv, SIGNAL( rightButtonPressed( QListViewItem *, const QPoint& , int ) ),
			this, SLOT( Menu(QListViewItem *, const QPoint& , int) ) );

		updateList();
	}
}

void ListDialog::updateList() {
	lv->clear();
	lv->setSorting(-1);
		
	GraphList *graphlist = p->getPlot(p->API())->getGraphList();
	for (int i= graphlist->getNumber()-1;i>=0;i--) {
		QStringList s;
		if (type == P2D) {
			GRAPHType st = graphlist->getStruct(i);
			if (st == GRAPH2D) {		// x-y
				Graph2D *g = graphlist->getGraph2D(i);
				s = g->Info();
			}
			else if (st == GRAPH3D) {	// x-y-dy
				lv->showZ();
				Graph3D *g = graphlist->getGraph3D(i);
				s = g->Info();
			}
			else if (st == GRAPH4D) {	// x-y-dx-dy, x-y-dy1-dy2
				lv->showZ();lv->showT();
				Graph4D *g = graphlist->getGraph4D(i);
				s = g->Info();
			}
		}
		else if (type == P3D ){
			GRAPHType st = graphlist->getStruct(i);
			if (st == GRAPH3D) {
				lv->showZ();
				Graph3D *g = graphlist->getGraph3D(i);
				s = g->Info();
			}
			else if (st == GRAPHM){
				GraphM *g = graphlist->getGraphM(i);
				s = g->Info();
			}
		}
		else if (type == PSURFACE) {
			GRAPHType st = graphlist->getStruct(i);
			if (st == GRAPHM) {
				GraphM *g = graphlist->getGraphM(i);
				s = g->Info();
			}
			else if (st == GRAPHIMAGE){
				GraphIMAGE *g = graphlist->getGraphIMAGE(i);
				s = g->Info();
			}
		}
		else if (type == PPIE || type == PPOLAR) {
			Graph2D *g = graphlist->getGraph2D(i);
			s = g->Info();
		}
		else if (type == PGRASS) {
			GraphGRASS *g = graphlist->getGraphGRASS(i);
			s = g->Info();
		}
		else if (type == PVTK) {
			GraphVTK *g = graphlist->getGraphVTK(i);
			s = g->Info();
		}
		else if (type == PTERNARY) {
			lv->showZ();
			Graph3D *g = graphlist->getGraph3D(i);
			s = g->Info();
		}
		else if (type == PQWT3D) {
			GRAPHType st = graphlist->getStruct(i);
			if (st == GRAPHM) {
				GraphM *g = graphlist->getGraphM(i);
				s = g->Info();
			}
			else if (st == GRAPH3D) {
				Graph3D *g = graphlist->getGraph3D(i);
				s = g->Info();
			}
		}
		QListViewItem *lvi = new QListViewItem(lv);
		for (unsigned int j = 0;j < s.count();j++) {
			lvi->setText(j,s[j]);
		}
	}
	
	// select at least last item
	lv->setSelected(lv->currentItem(),TRUE);
	
	p->getMainWin()->setModified();
}

// toggle shown of selected graph
void ListDialog::toggleShown() {
	QListViewItemIterator it(lv);
	for ( ; it.current(); ++it ) {
		if(it.current()->isSelected()) {
			int item = (int) (lv->itemPos(it.current())/it.current()->height());

			Graph *g = p->getPlot(p->API())->getGraphList()->getGraph(item);
			if(g->isShown())
				g->setShown(FALSE);
			else
				g->setShown(TRUE);
		}
	}
	
	updateList();
	p->updatePixmap();
	p->resetRanges();
}

// delete selected graph
void ListDialog::deleteGraph() {
//	kdDebug()<<"ListDialog::deleteGraph()"<<endl;
	QListViewItemIterator it(lv);
	int cur_item=0;
	for ( ; it.current(); ++it ) {
		if(it.current()->isSelected()) {
			int item = (int) (lv->itemPos(it.current())/it.current()->height());
			// shift item after deleted items
			p->getPlot(p->API())->getGraphList()->delGraph(item - cur_item++);
		}
	}
	
	p->resetRanges();

	updateList();
	p->updatePixmap();
}

void ListDialog::changeGraph() {
	Plot *plot = p->getPlot(p->API());
	PType type = plot->Type();
	
	QListViewItemIterator it(lv);
	for ( ; it.current(); ++it ) {
		if(it.current()->isSelected()) {
			int item = (int) (lv->itemPos(it.current())/it.current()->height());
			QString name = it.current()->text(0);

			if(type == PVTK) 
				(new VTKDialog(mw,name,item))->show();
			else if (type == PGRASS)
				(new GRASSDialog(mw,name,item))->show();
			else {	// everything else
				if(plot->getGraphList()->getGraph(item)->Source()==SDATA)		// data
					(new DataDialog(mw,name,this,item,plot->Type()))->show();
				else		// function
					(new FunctionDialog(mw,name,this,item,plot->Type()))->show();
			}
		}
	}
}

//! dump the selected graph data to a file
void ListDialog::dumpGraph() {
	QListViewItemIterator it(lv);
	for ( ; it.current(); ++it ) {
		if(it.current()->isSelected()) {
			int item = (int) (lv->itemPos(it.current())/it.current()->height());
			QString name = it.current()->text(0);
			(new DumpDialog(p->getMainWin(),name,item))->show();
		}
	}
}

//! edit the selected graph data
void ListDialog::editGraph() {
	QListViewItemIterator it(lv);
	for ( ; it.current(); ++it ) {
		if(it.current()->isSelected()) {
			int item = (int) (lv->itemPos(it.current())/it.current()->height());
			QString name = it.current()->text(0);
			
			if(p->getPlot(p->API())->getGraphList()->getStruct(item) == GRAPHIMAGE)
				continue;	// image not editable
			
			(new EditDialog(p,name,item,this))->show();
		}
	}
}

void ListDialog::openSpreadsheet() {
	QListViewItemIterator it(lv);
	for ( ; it.current(); ++it ) {
		if(it.current()->isSelected()) {
			int item = (int) (lv->itemPos(it.current())/it.current()->height());

			GraphList *gl = p->getPlot(p->API())->getGraphList();
			GRAPHType s = gl->getStruct(item);

			Spreadsheet *spreadsheet = p->getMainWin()->newSpreadsheet();
			QTable *table = spreadsheet->Table();
			if(s != GRAPHIMAGE)	// here Number = NX*NY !
				table->setNumRows(gl->getGraph(item)->Number());


			if (s == GRAPH2D) {
				Graph2D *g = gl->getGraph2D(item);
				Point *data = g->Data();
				for(int i=0;i<g->Number();i++) {
					if(data[i].Masked()) {
						LTableItem *itx, *ity;
						itx = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].X()),Qt::red);
						ity = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].Y()),Qt::red);
						itx->setWordWrap(true);
						ity->setWordWrap(true);
						table->setItem( i, 0, itx);
						table->setItem( i, 1, ity);
					}
					else {
						table->setText(i,0,QString::number(data[i].X()));
						table->setText(i,1,QString::number(data[i].Y()));
					}
				}
			}
			else if (s == GRAPH3D) {
				Graph3D *g = gl->getGraph3D(item);
				Point3D *data = g->Data();
				table->setNumCols(3);
				table->horizontalHeader()->setLabel( 2, QString( "C [Z]" ) );
				for(int i=0;i<g->Number();i++) {
					if(data[i].Masked()) {
						LTableItem *itx, *ity, *itz;
						itx = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].X()),Qt::red);
						ity = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].Y()),Qt::red);
						itz = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].Z()),Qt::red);
						itx->setWordWrap(true);
						ity->setWordWrap(true);
						itz->setWordWrap(true);
						table->setItem( i, 0, itx);
						table->setItem( i, 1, ity);
						table->setItem( i, 2, itz);
					}
					else {
						table->setText(i,0,QString::number(data[i].X()));
						table->setText(i,1,QString::number(data[i].Y()));
						table->setText(i,2,QString::number(data[i].Z()));
					}
				}
			}
			else if (s == GRAPHM) {
				GraphM *g = gl->getGraphM(item);
				double *data = g->Data();
				table->setNumCols(g->NX());
				table->setNumRows(g->NY());
				// set label for all columns
				for (int i=1;i<table->numCols();i++)
					table->horizontalHeader()->setLabel( i, QChar(i+65)+QString( " [Y]" ) );
				
				// TODO : set masked data
				for(int i=0;i<g->NY();i++)
					for(int j=0;j<g->NX();j++)
						table->setText(i,j,QString::number(data[j+g->NX()*i]));
			}
			else if (s == GRAPH4D) {
				Graph4D *g = gl->getGraph4D(item);
				Point4D *data = g->Data();
				table->setNumCols(4);
				table->horizontalHeader()->setLabel( 2, QString( "C [Y]" ) );
				table->horizontalHeader()->setLabel( 3, QString( "D [Y]" ) );
				for(int i=0;i<g->Number();i++) {
					if(data[i].Masked()) {
						kdDebug()<<"Point "<<i<<" MASKED ("<<data[i].Masked()<<')'<<endl;
						LTableItem *itx, *ity, *itz, *itt;
						itx = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].X()),Qt::red);
						ity = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].Y()),Qt::red);
						itz = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].Z()),Qt::red);
						itt = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].T()),Qt::red);
						itx->setWordWrap(true);
						ity->setWordWrap(true);
						itz->setWordWrap(true);
						itt->setWordWrap(true);
						table->setItem( i, 0, itx);
						table->setItem( i, 1, ity);
						table->setItem( i, 2, itz);
						table->setItem( i, 3, itz);
					}
					else {
						table->setText(i,0,QString::number(data[i].X()));
						table->setText(i,1,QString::number(data[i].Y()));
						table->setText(i,2,QString::number(data[i].Z()));
						table->setText(i,3,QString::number(data[i].T()));
					}
				}
			}
			else if (s == GRAPHIMAGE) {
				GraphIMAGE *g = gl->getGraphIMAGE(item);
				QPixmap pm = g->Pixmap();
				table->setNumCols(pm.width());
				table->setNumRows(pm.height());
				// set label for all columns
				for (int i=1;i<table->numCols();i++)
					table->horizontalHeader()->setLabel( i, QChar(i+65)+QString( " [Y]" ) );
				
				QImage image = pm.convertToImage();
				
				for (int i=0;i<pm.width();i++) {
					for (int j=0;j<pm.height();j++) {
						table->setText(j,i,QString::number(qGray(image.pixel(i,j))));
					}
				}
			}
		}
	}
	MainWin *mw = p->getMainWin();
	mw->setActiveSheet(mw->NrWorksheets()+mw->NrSpreadsheets()-1);
}

//! show selected graph info and statistics
void ListDialog::statGraph() {
	QListViewItemIterator it(lv);
	for ( ; it.current(); ++it ) {
		if(it.current()->isSelected()) {
			int item = (int) (lv->itemPos(it.current())/it.current()->height());
			QString name = it.current()->text(0);
			
			(new InfoDialog(p,name,item))->show();
		}
	}
}

//! toggle masking of selected graphs
void ListDialog::toggleMask() {
	GraphList *gl = p->getPlot(p->API())->getGraphList();
	if(gl->getNumber()==0)
		return;

	QListViewItemIterator it(lv);
	for ( ; it.current(); ++it ) {
		if(it.current()->isSelected()) {
			int item = (int) (lv->itemPos(it.current())/it.current()->height());
			GRAPHType s = gl->getStruct(item);
			switch(s) {
			case GRAPH2D: {
				Graph2D *g = gl->getGraph2D(item);
				Point *d = g->Data();
				for(int i=0;i<g->Number();i++) {
					if(d[i].Masked())
						d[i].setMasked(FALSE);
					else
						d[i].setMasked(TRUE);
				}
				}; break;
			case GRAPH3D: {
				Graph3D *g = gl->getGraph3D(item);
				Point3D *d = g->Data();
				for(int i=0;i<g->Number();i++) {
					if(d[i].Masked())
						d[i].setMasked(FALSE);
					else
						d[i].setMasked(TRUE);
				}
				}; break;
			case GRAPH4D: {
				Graph4D *g = gl->getGraph4D(item);
				Point4D *d = g->Data();
				for(int i=0;i<g->Number();i++) {
					if(d[i].Masked())
						d[i].setMasked(FALSE);
					else
						d[i].setMasked(TRUE);
				}
				}; break;
			case GRAPHM: {
				//TODO
				}; break;
			default: break;
			}
		}
	}

	p->updatePixmap();
}

//! toggle masking of selected graphs
void ListDialog::unMask() {
	GraphList *gl = p->getPlot(p->API())->getGraphList();
	if(gl->getNumber()==0)
		return;
	
	QListViewItemIterator it(lv);
	for ( ; it.current(); ++it ) {
		if(it.current()->isSelected()) {
			int item = (int) (lv->itemPos(it.current())/it.current()->height());
			GRAPHType s = gl->getStruct(item);
			switch(s) {
			case GRAPH2D: {
				Graph2D *g = gl->getGraph2D(item);
				Point *d = g->Data();
				for(int i=0;i<g->Number();i++)
					d[i].setMasked(FALSE);
				}; break;
			case GRAPH3D: {
				Graph3D *g = gl->getGraph3D(item);
				Point3D *d = g->Data();
				for(int i=0;i<g->Number();i++)
					d[i].setMasked(FALSE);
				}; break;
			case GRAPH4D: {
				Graph4D *g = gl->getGraph4D(item);
				Point4D *d = g->Data();
				for(int i=0;i<g->Number();i++)
						d[i].setMasked(FALSE);
				}; break;
			case GRAPHM: {
				//TODO
				}; break;
			default: break;
			}
		}
	}
	
	p->updatePixmap();
}

//! clone selected graphs
void ListDialog::Clone() {
	Plot *plot = p->getPlot(p->API());
	GraphList *gl = plot->getGraphList();
	if(gl->getNumber()==0)
		return;
	
	int item = (int) (lv->itemPos(lv->currentItem())/lv->currentItem()->height());

	GRAPHType s = gl->getStruct(item);
	switch(s) {
	case GRAPH2D: {
		Graph2D *g = new Graph2D(*(gl->getGraph2D(item)));
		gl->addGraph2D(g);
		}; break;
	case GRAPH3D: {
		Graph3D *g = new Graph3D(*(gl->getGraph3D(item)));
		gl->addGraph3D(g);
		}; break;
	case GRAPH4D: {
		Graph4D *g = new Graph4D(*(gl->getGraph4D(item)));
		gl->addGraph4D(g);
		}; break;
	case GRAPHM: {
		GraphM *g = new GraphM(*(gl->getGraphM(item)));
		gl->addGraphM(g);
		}; break;
	case GRAPHIMAGE: {
		GraphIMAGE *g = new GraphIMAGE(*(gl->getGraphIMAGE(item)));
		gl->addGraphIMAGE(g);
		}; break;
	case GRAPHGRASS: {
		GraphGRASS *g = new GraphGRASS(*(gl->getGraphGRASS(item)));
		gl->addGraphGRASS(g);
		}; break;
	default:
		break;
	}

	updateList();
	p->updatePixmap();
}

// update range (ale-ble) when selecting another graph
void ListDialog::updateRange() {
	GraphList *gl = p->getPlot(p->API())->getGraphList();
	if(gl->getNumber()==0)
		return;
	int item = (int) (lv->itemPos(lv->currentItem())/lv->currentItem()->height());

	GRAPHType s = gl->getStruct(item);
	LRange range=0;

	if (s == GRAPH2D) {
		Graph2D *g = gl->getGraph2D(item);
		range = g->Range(0);
		ale->setText(QString::number(range.rMin()));
		ble->setText(QString::number(range.rMax()));
	}
}
