//LabPlot : PeakListDialog.cc

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <qstring.h>
#include <qlabel.h>
#include <qfiledialog.h>
#include <qcolordialog.h>
#include <klocale.h>
#include "PeakListDialog.h"
#include "source.h"

using namespace std;

PeakListDialog::PeakListDialog(MainWin *mw, const char *name)
	: ListDialog(mw, name)
{
	setCaption(i18n("Peak Dialog"));

	QTabWidget *tw = new QTabWidget(vbox);
	QVBox *tab1 = new QVBox(tw);

	new QLabel(i18n(" Threshold (y range) :"),tab1);
	QHBox *hb = new QHBox(tab1);
	new QLabel(QString(" dy = "),hb);
	thresholdle = new KLineEdit(QString("0.5"),hb);
	thresholdle->setValidator(new QDoubleValidator(thresholdle));

	new QLabel(i18n(" Accuracy (x range) :"),tab1);
	hb = new QHBox(tab1);
	new QLabel(QString(" x index = "),hb);
	accuracyle = new KLineEdit(QString("3"),hb);
	accuracyle->setValidator(new QIntValidator(1,INF,accuracyle));

	hb = new QHBox(tab1);
	negativecb = new QCheckBox(i18n("negative peaks "),hb);
	negativecb->setChecked(false);
	
	Style *style=0;
	Symbol *symbol=0;
	QVBox *styletab;
	if(p && p->getPlot(p->API())->Type() == PSURFACE)
		styletab = surfaceStyle(tw,TRUE);
	else
		styletab = simpleStyle(tw,0, style, symbol);

	tw->addTab(tab1,i18n("Parameter"));
	tw->addTab(styletab,i18n("Style"));

	QObject::connect(ok,SIGNAL(clicked()),SLOT(ok_clicked()));
        QObject::connect(apply,SIGNAL(clicked()),SLOT(apply_clicked()));
	
	setMinimumWidth(vbox->minimumSizeHint().width());
	setMinimumHeight(gbox->minimumSizeHint().height()+vbox->minimumSizeHint().height());
	resize(minimumSize());
}

void PeakListDialog::apply_clicked() {
	// TODO : more selected graphs ???
	int n = 0;//(nle->text()).toInt();
	double threshold = thresholdle->text().toDouble();
	double accuracy = accuracyle->text().toInt();
	
	if(s) {
		QTable *table = s->Table();
		int col = table->currentColumn();
		
		// add column if needed
		bool empty=TRUE;
		for(int i=0;i<table->numRows();i++) {
			if(table->text(i,table->numCols()-1).length()>0) {
				empty =FALSE;
			}
		}
		if (!empty)
			s->addColumn();

		int newcol = table->numCols()-1;
		// TODO !
		for (int i = 0;i<table->numRows()/n;i++) {
			double sum = 0;
			for (int j=0;j<n;j++) {
				sum+=table->text(n*i+j,col).toDouble();
			}
			sum = sum/n;
			
			// insert into last column
			table->setText(n*i+n/2,newcol,QString::number(sum));
		}
		return;
	}
	
	GraphList *gl = p->getPlot(p->API())->getGraphList();
	if(gl->getNumber()==0)
		return;
	int item = (int) (lv->itemPos(lv->currentItem())/lv->currentItem()->height());
	GRAPHType s = gl->getStruct(item);

	if (s == GRAPH2D) {
		Graph2D *g = gl->getGraph2D(item);
		int nx = g->Number();
		Point *ptr = new Point[nx];
		Point *a = g->Data();

		// do it
		double xmin=0, xmax=1, ymin=0, ymax=1;
		int nr_points=0;
		for (int i = 0;i<nx;i++) {
			bool peak_found=FALSE;
			double x = 0;
			double y = 0;
			
			for (int j=1;j<accuracy;j++) {
				if(negativecb->isChecked()) {
					if ((a[i].Y() < a[i-j].Y()-threshold || a[i].Y() < a[i+j].Y()-threshold )
						&& a[i].Y() < a[i-1].Y() && a[i].Y() < a[i+1].Y())
							peak_found=TRUE;
				}
				else {
					if ((a[i].Y() > a[i-j].Y()+threshold || a[i].Y() > a[i+j].Y()+threshold )
						&& a[i].Y()>a[i-1].Y() && a[i].Y()>a[i+1].Y())
							peak_found=TRUE;
				}
			}
			
			if(peak_found) {
				x = a[i].X();
				y = a[i].Y();
				nr_points++;
				kdDebug()<<"peak found ! @ "<<x<<'/'<<y<<endl;

				// new ranges
				if (nr_points==1) {
					xmin=xmax=x;
					ymin=ymax=y;
				}
				else {
					x<xmin?xmin=x:0;
					x>xmax?xmax=x:0;
					y<ymin?ymin=y:0;
					y>ymax?ymax=y:0;
				}

				ptr[nr_points-1].setPoint(x,y);
			}
		}

		// create the new Graph
		LRange range[2];
		range[0] = LRange(xmin,xmax);
		range[1] = LRange(ymin,ymax);

		QString fun = QString(i18n("peaks (")+QString::number(nr_points)+i18n(") of ")+g->Label());

		Style *style = new Style(cb2->currentItem(),color->color(),filled->isChecked(),fcolor->color(),
			widthle->text().toInt(),pencb->currentItem(),brushcb->currentItem());
		Symbol *symbol = new Symbol((SType)symbolcb->currentItem(),scolor->color(),
			ssize->text().toInt(),(FType)symbolfillcb->currentItem(),sfcolor->color(),sbrushcb->currentItem());
		
		Graph2D *ng = new Graph2D(fun,fun,range,SSPREADSHEET,P2D,style,symbol,ptr,nr_points);
		mw->addGraph2D(ng,sheetcb->currentItem());
	}
	else if (s == GRAPH3D) {
		Graph3D *g = gl->getGraph3D(item);
		int nx = g->Number();
		Point3D *ptr = new Point3D[nx];
		Point3D *a = g->Data();

		cout<< "TYPE : "<<(int)type<<endl;
		
		// do the average
		double xmin=0, xmax=1, ymin=0, ymax=1, zmin=0, zmax=1;
		for (int i = 0;i<nx/n;i++) {
			double x = 0, y = 0, z = 0;
			if (type == P2D) {	// 2derr
				x = a[(int) (n*i+n/2)].X();
				for (int j=0;j<n;j++) {
					y += a[n*i+j].Y();
					z += a[n*i+j].Z();
				}
				y = y/n;
				z = z/n;
			}
			else {		// 3d
				for (int j = 0;j<n;j++) {
					x+=a[(int)(j+n*i)].X();
					y+=a[(int)(j+n*i)].Y();
					z+=a[(int)(j+n*i)].Z();
				}
				x = x/n;
				y = y/n;
				z = z/n;
			}
			
			// new ranges
			if (i==0) {
				xmin=xmax=x;
				ymin=ymax=y;
				zmin=zmax=z;
			}
			else {
				x<xmin?xmin=x:0;
				x>xmax?xmax=x:0;
				y<ymin?ymin=y:0;
				y>ymax?ymax=y:0;
				z<zmin?zmin=z:0;
				z>zmax?zmax=z:0;
			}

			ptr[i].setPoint(x,y,z);
		}

		// create the new Graph
		LRange range[3];
		range[0] = LRange(xmin,xmax);
		range[1] = LRange(ymin,ymax);
		range[2] = LRange(zmin,zmax);
		
		QString fun = QString(i18n("average (")+QString::number(n)+i18n(") of ")+g->Label());
		
		Style *style = new Style(cb2->currentItem(),color->color(),filled->isChecked(),fcolor->color(),
			widthle->text().toInt(),pencb->currentItem(),brushcb->currentItem());
		Symbol *symbol = new Symbol((SType)symbolcb->currentItem(),scolor->color(),ssize->text().toInt(),
			(FType)symbolfillcb->currentItem(),sfcolor->color(),sbrushcb->currentItem());

		Graph3D *ng = new Graph3D(fun,fun,range,SSPREADSHEET,type,style,symbol,ptr,g->Number()/n,1);
		ng->setNumberX((int)(g->NX()/n));
		ng->setNumberY((int)(g->NY()));
		mw->addGraph3D(ng,sheetcb->currentItem());

	}
	else if (s == GRAPHM) {		// PSURFACE and P3D
		GraphM *g = gl->getGraphM(item);
		int N = g->Number();
		int nx = g->NX();
		int ny = g->NY();
		double *ptr = new double[N];
		double *a = g->Data();

		// do the average
		double zmin=0, zmax=1;
		for (int i = 0;i<nx/n;i++) {
			for (int j = 0;j<ny/n;j++) {
				double z = 0;
		
				for (int l=0;l<n;l++) {
					int a1 = n*i+l;
					for (int k=0;k<n;k++) {
						int a2 = n*j+k;
						z += a[a2+nx*a1];
					}
				}
				z=z/(n*n);
				// new ranges
				if (i==0) {
					zmin=zmax=z;
				}
				else {
					z<zmin?zmin=z:0;
					z>zmax?zmax=z:0;
				}

				ptr[j+(nx/n)*i]=z;
			}
			cout<<endl;
		}

		// create the new Graph
		LRange range[3];
		range[0] = LRange(1,nx);
		range[1] = LRange(1,ny);
		range[2] = LRange(zmin,zmax);
		
		QString fun = QString(i18n("average (")+QString::number(n)+i18n(") of ")+g->Label());
		
		Style *style = new Style(cb2->currentItem(),color->color(),filled->isChecked(),fcolor->color(),
			widthle->text().toInt(),pencb->currentItem(),brushcb->currentItem());
		Symbol *symbol = new Symbol((SType)symbolcb->currentItem(),scolor->color(),ssize->text().toInt(),
			(FType)symbolfillcb->currentItem(),sfcolor->color(),sbrushcb->currentItem());
		
		GraphM *ng = new GraphM(fun,fun,range,SSPREADSHEET,type,style,symbol,ptr,nx/n,ny/n);
		mw->addGraphM(ng,sheetcb->currentItem(),type);
	}

	updateList();
}
