//LabPlot : PlotDialog.cc

#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <qlabel.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qcolordialog.h>
#include <qbuttongroup.h>
#include <klocale.h>
#include "PlotDialog.h"
#include "Plot2DSimple.h"
#include "Plot2DSurface.h"
#include "Plot3D.h"
#include "PlotPie.h"
#include "PlotPolar.h"
#include "PlotGRASS.h"
#include "PlotVTK.h"
#include "PlotTernary.h"
#include "PlotGL.h"
#include "PlotQWT3D.h"

//! general class for plot settings
PlotDialog::PlotDialog(MainWin *mw, const char *name)
	: Dialog(mw, name)
{
	setCaption(i18n("Plot Dialog")+i18n(" : ")+QString(name));

	lv = new QListView(vbox,0);
	lv->addColumn(i18n("Number"));
	lv->addColumn(i18n("Type"));
	lv->addColumn(i18n("Position"));
	lv->addColumn(i18n("Size"));
	lv->addColumn(i18n("Transparent"));
	lv->addColumn(i18n("Background Color"));
	lv->addColumn(i18n("Graph Background Color"));

	lv->setAllColumnsShowFocus(true);
	for (int i=0;i<lv->columns();i++)
		lv->setColumnAlignment(i,Qt::AlignHCenter);
	// multi selection with CTRL and SHIFT
	lv->setSelectionMode(QListView::Extended);
	lv->setMinimumWidth(350);

	// popup menu
	if(p) {
		menu = new QPopupMenu( lv );
		menu->insertItem( i18n( "Clone" ),this,SLOT(clonePlot()) );
		menu->insertItem( i18n( "Delete" ),this,SLOT(deletePlot()) );
		connect(lv, SIGNAL( rightButtonPressed( QListViewItem *, const QPoint& , int ) ),
			this, SLOT( Menu(QListViewItem *, const QPoint& , int) ) );
	}
	updateList();
}

void PlotDialog::updateList() {
	lv->clear();
	lv->setSorting(-1);

	for (int i= p->NrPlots()-1;i>=0;i--) {
		QListViewItem *lvi = new QListViewItem(lv);
		QStringList s;
		s += QString::number(i+1);

		Plot *plot = p->getPlot(i);
		PType type = plot->Type();
		switch (type) {
		case P2D:	s += ((Plot2DSimple *)plot)->Info();break;
		case PSURFACE:	s += ((Plot2DSurface *)plot)->Info();break;
		case P3D:	s += ((Plot3D *)plot)->Info();break;
		case PGRASS:	s += ((PlotGRASS *)plot)->Info();break;
		case PVTK:	s += ((PlotVTK *)plot)->Info();break;
		case PPIE:	s += ((PlotPie *)plot)->Info();break;
		case PPOLAR:	s += ((PlotPolar *)plot)->Info();break;
		case PTERNARY:	s += ((PlotTernary *)plot)->Info();break;
		case PGL: 	s += ((PlotGL *)plot)->Info();break;
		case PQWT3D:	s += ((PlotQWT3D *)plot)->Info();break;
		}
			
		for (unsigned int j = 0;j < s.count();j++) {
			lvi->setText(j,s[j]);
		}
	}
}

void PlotDialog::deletePlot() {
	QListViewItemIterator it(lv);
	for ( ; it.current(); ++it ) {
		if(it.current()->isSelected()) {
			int item = (int) (lv->itemPos(it.current())/it.current()->height());
			p->deletePlot(item);
		}
	}

	updateList();
}

void PlotDialog::clonePlot() {
	QListViewItemIterator it(lv);
	for ( ; it.current(); ++it ) {
		if(it.current()->isSelected()) {
			int pitem = (int) (lv->itemPos(it.current())/it.current()->height());
			Plot *plot = p->getPlot(pitem);
			p->newPlot(plot->Type());
			Plot *newplot = p->getPlot(p->NrPlots()-1);
			
			// copy ranges
			newplot->setRanges(plot->Ranges());
			newplot->setActRanges(plot->ActRanges());
			
			// clone all graphs
			GraphList *gl = plot->getGraphList();
			GraphList *newgl = newplot->getGraphList();
			for(unsigned int item=0;item<gl->getNumber();item++) {
				GRAPHType s = gl->getStruct(item);
				switch(s) {
				case GRAPH2D: {
					Graph2D *g = new Graph2D(*(gl->getGraph2D(item)));
					newgl->addGraph2D(g);
					}; break;
				case GRAPH3D: {
					Graph3D *g = new Graph3D(*(gl->getGraph3D(item)));
					newgl->addGraph3D(g);
					}; break;
				case GRAPH4D: {
					Graph4D *g = new Graph4D(*(gl->getGraph4D(item)));
					newgl->addGraph4D(g);
					}; break;
				case GRAPHM: {
					GraphM *g = new GraphM(*(gl->getGraphM(item)));
					newgl->addGraphM(g);
					}; break;
				case GRAPHIMAGE: {
					GraphIMAGE *g = new GraphIMAGE(*(gl->getGraphIMAGE(item)));
					newgl->addGraphIMAGE(g);
					}; break;
				case GRAPHGRASS: {
					GraphGRASS *g = new GraphGRASS(*(gl->getGraphGRASS(item)));
					newgl->addGraphGRASS(g);
					}; break;
				}
			}
		}
	}
	
	updateList();
	p->updatePixmap();
}
